@@ -986,21 +986,23 @@ option `gnorb-gnus-hint-relevant-article' is non-nil."
 (add-hook 'gnus-select-article-hook 'gnorb-gnus-hint-relevant-message)
 
 (defun gnorb-gnus-insert-format-letter-maybe (header)
-  (if (not (or (gnus-ephemeral-group-p gnus-newsgroup-name)
-	       (gnus-virtual-group-p gnus-newsgroup-name)))
-      (let* ((id (mail-header-message-id header))
-	     ;; Use lower-level accessor to avoid creating an entry
-	     ;; where there wasn't one.  This function doesn't respect
-	     ;; ignored registry groups.
-	     (entry (nth 1 (assoc id (registry-lookup
-				      gnus-registry-db
-				      (list id))))))
-	(cond ((cdr-safe (assq 'gnorb-ids entry))
-	       gnorb-gnus-summary-tracked-mark)
-	      ((gnorb-find-tracked-headings header)
-	       gnorb-gnus-summary-mark)
-	      (t " ")))
-    " "))
+  (if (hash-table-p gnus-registry-db)
+      (if (not (or (gnus-ephemeral-group-p gnus-newsgroup-name)
+		   (gnus-virtual-group-p gnus-newsgroup-name)))
+	  (let* ((id (mail-header-message-id header))
+		 ;; Use lower-level accessor to avoid creating an entry
+		 ;; where there wasn't one.  This function doesn't respect
+		 ;; ignored registry groups.
+		 (entry (nth 1 (assoc id (registry-lookup
+					  gnus-registry-db
+					  (list id))))))
+	    (cond ((cdr-safe (assq 'gnorb-ids entry))
+		   gnorb-gnus-summary-tracked-mark)
+		  ((gnorb-find-tracked-headings header)
+		   gnorb-gnus-summary-mark)
+		  (t " ")))
+	" ")
+    ""))
 
 (defalias (intern (concat "gnus-user-format-function-"
 			  gnorb-gnus-summary-mark-format-letter))
@@ -1008,15 +1010,17 @@ option `gnorb-gnus-hint-relevant-article' is non-nil."
     (gnorb-gnus-insert-format-letter-maybe header)))
 
 (defun gnorb-gnus-insert-format-tags (header)
-  (let* ((id (mail-header-message-id header))
-	 (entry (nth 1 (assoc id (registry-lookup
-				  gnus-registry-db
-				  (list id)))))
-	 (tags (cdr-safe (assq 'org-tags entry))))
-    (if tags
-	(concat
-	 ":" (mapconcat #'identity tags ":") ":")
-      "")))
+  (if (hash-table-p gnus-registry-db)
+      (let* ((id (mail-header-message-id header))
+	     (entry (nth 1 (assoc id (registry-lookup
+				      gnus-registry-db
+				      (list id)))))
+	     (tags (cdr-safe (assq 'org-tags entry))))
+	(if tags
+	    (concat
+	     ":" (mapconcat #'identity tags ":") ":")
+	  ""))
+    ""))
 
 (defalias (intern (concat "gnus-user-format-function-"
 			  gnorb-gnus-summary-tags-format-letter))
